/* gno3dtet: 3D Tetris game for GNOME
 * Copyright (C) 1999 Sebastien Nicoud <snicoud@rmi.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
*/

/* Indent TAB = 4
*/

#ifndef _COMMON
#define _COMMON

// includes
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include <gnome.h>
#include <iostream.h>
#include <fstream.h>

// general constants
#define GNO3DTET_NAME           "Gnome 3D Tetris"
#define GNO3DTET_AUTHOR		    "Sebastien Nicoud"
#define GNO3DTET_EMAIL		    "<snicoud@rmi.net>, <snicoud@hotmail.com>"
#define GNO3DTET_URL			"http://webdat.com/seb/3dtetris.html"
#define GNO3DTET_DATECREATED	"10/25/1999"
#define GNO3DTET_DATEMODIFIED	"11/17/1999"
#define GNO3DTET_EXENAME 	    "gno3dtet"
#define GNO3DTET_VERSION		"0.3.2"
#define GNO3DTET_HOFABSFILE 	HOFDIR HOFFILE
		// defined in the Makefile, passed by gcc -D :
		// HOFDIR = 			"$(prefix)/var/games/"
		// HOFFILE = 			"gno3dtet.hof"
#define LICENSE_FILE    		"copying.html"
#define SAVED_GAME				".gno3dtet.saved"
#define SAVED_SESSION			".gno3dtet.session"
#define CONFIG_PREFERENCES		"Preferences"
#define CONFIG_NHOFL			"Nb of lines in HOF"
#define CONFIG_DIFF				"Difficulty"
#define CONFIG_WIDTH			"World width"
#define CONFIG_HEIGHT			"World height"
#define CONFIG_DEPTH			"World depth"
#define CONFIG_SHOW_TOOLBAR		"Show toolbar"
#define CONFIG_SHOW_SCALE		"Show scale"
#define CONFIG_SHOW_SCORE		"Show score"
#define CONFIG_SHOW_STATS		"Show stats"
#define CONFIG_SHOW_NEXT_PIECE	"Show next piece"

#define MAX_UINT64 G_GINT64_CONSTANT(0xFFFFFFFFFFFFFFFFU)

// application constants
#define MAIN_TIMER				30
#define CUBE_PARCOURU_EN_1K_MSEC 0.0005
#define INCREMENT_LEVEL_EVERY_N_LAYERS	5
#define	MIN_WORLD_WIDTH			3
#define DEF_WORLD_WIDTH			5
#define	MAX_WORLD_WIDTH			10
#define	MIN_WORLD_HEIGHT		3
#define DEF_WORLD_HEIGHT		5
#define	MAX_WORLD_HEIGHT		10
#define	MIN_WORLD_DEPTH			5
#define DEF_WORLD_DEPTH			10
#define	MAX_WORLD_DEPTH			20
#define DEF_DIFFICULTY			3
#define MAX_CUBES_IN_PIECE		5
#define MIN_NHOFL				1
#define MAX_NHOFL 				50
#define DFLT_NHOFL				10
#define MAX_NAME_LEN			32
#define MAX_HOF_S_LEN			32
#define SCALE_WIDTH				20
#define STATS_WIDTH				30
#define NB_SCORE_DIGITS			9
#define NB_LEVEL_DIGITS			3
#define NB_LAYERS_DIGITS		4
#define NB_BLOCS_DIGITS			5
#define NB_CUBES_DIGITS			6
#define	BIG_TEXT_HEIGHT			12
#define SMALL_TEXT_HEIGHT		7
#define SCORE_TEXT_WIDTH		41
#define	LEVEL_TEXT_WIDTH		41
#define	STATS_TEXT_WIDTH		25
#define	BIG_DIGIT_HEIGHT		22
#define	BIG_DIGIT_WIDTH			12
#define	BIG_COMMA_WIDTH			4
#define SMALL_DIGIT_HEIGHT		13
#define SMALL_DIGIT_WIDTH		7
#define	SMALL_COMMA_WIDTH		3

// debug stuff
#define TRACE { if (debug) printf("trace %s @ %u\n",__FILE__,__LINE__); }
#define DBG_CB					1

// includes
#include "config.h"
//#include "cpiece.h"
//#include "csettings.h"
//#include "cgame.h"
//#include "cwsettings.h"
//#include "cwmain.h"

// classes
class CSettings;
class CWSettings;
class CPiece;
class CGame;
class CWMain;

// common vars
extern gint debug; // debug level
extern CSettings *settings; // official settings
extern CWMain *mwin; // main window
extern CGame *mds; // main data structure
extern gint	g_nhofl; // nb of line displayed in the HOF

// common functions


#endif //_COMMON
