/* gno3dtet: 3D Tetris game for GNOME
 * Copyright (C) 1999 Sebastien Nicoud <snicoud@rmi.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
*/

#include "common.h"
#include "csettings.h"

const gchar *CSettings::str_width  = "width";
const gchar *CSettings::str_height = "height";
const gchar *CSettings::str_depth  = "depth";
const gchar *CSettings::str_diff   = "difficulty";

const gchar *CSettings::diff_descriptors[5] = {
	"easiest", "Easy", "Medium", "Hard", "hardest"
};

CSettings::CSettings()
{
	if (debug) g_print("CSettings::CSettings()\n");
	_w = DEF_WORLD_WIDTH;
	_h = DEF_WORLD_HEIGHT;
	_d = DEF_WORLD_DEPTH;
	compute_total_cubes();
	_difficulty = diff_HARD;
}

CSettings::CSettings(const CSettings &ind)
{
	if (debug) g_print("CSettings::CSettings(CSettings &)\n");
	_w = ind._w;
	_h = ind._h;
	_d = ind._d;
	compute_total_cubes();
	_difficulty = ind._difficulty;
}

CSettings::~CSettings()
{
	if (debug) g_print("CSettings::~CSettings\n");
}

CSettings &CSettings::operator= (const CSettings &ind)
{
	if (debug > 2) g_print("CSettings::operator=\n");
	_w = ind._w;
	_h = ind._h;
	_d = ind._d;
	compute_total_cubes();
	_difficulty = ind._difficulty;
	return *this;
}

ostream &operator<< (ostream &os, CSettings &ind)
{
	if (debug > 2) g_print("CSettings::operator<<\n");
	cout << "settings: ";
	cout <<	CSettings::str_width << " = " << ind._w;
	cout << ", " << CSettings::str_height << " = " << ind._h;
	cout << ", " << CSettings::str_depth << " = " << ind._d;
	cout << ", total_cubes = " << ind._total_cubes << endl;
	cout << "  " << CSettings::str_diff << " = ";
	cout << CSettings::diff_descriptors[ind._difficulty] << endl;
}
