/* gno3dtet: 3D Tetris game for GNOME
 * Copyright (C) 1999 Sebastien Nicoud <snicoud@rmi.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
*/

#include "common.h"
#include "csettings.h"
#include "cwsettings.h"

CWSettings::CWSettings()
{
	if (debug) g_print("CWSettings::CWSettings()\n");
	_parent = NULL;
	_settings = new CSettings();
}

CWSettings::~CWSettings()
{
	if (debug) g_print("CWSettings::~CWSettings()\n");
	free_memory();
}

void CWSettings::apply(void)
{
	if (debug) g_print("CWSettings::apply()\n");
	settings->_w = _settings->_w;
	settings->_h = _settings->_h;
	settings->_d = _settings->_d;
	settings->_difficulty = _settings->_difficulty;
	// save the settings in ini file
	gnome_config_set_int(GNO3DTET_EXENAME"/"CONFIG_PREFERENCES"/"CONFIG_WIDTH,
		settings->_w);
	gnome_config_set_int(GNO3DTET_EXENAME"/"CONFIG_PREFERENCES"/"CONFIG_HEIGHT,
		settings->_h);
	gnome_config_set_int(GNO3DTET_EXENAME"/"CONFIG_PREFERENCES"/"CONFIG_DEPTH,
		settings->_d);
	gnome_config_set_int(GNO3DTET_EXENAME"/"CONFIG_PREFERENCES"/"CONFIG_DIFF,
		settings->_difficulty);
	gnome_config_sync();
}

void CWSettings::change_depth(void)
{
	if (debug > 1) g_print("CWSettings::change_depth\n");
	gnome_property_box_changed (GNOME_PROPERTY_BOX(_window));
	_settings->_d = ((gint)GTK_ADJUSTMENT(_adj_depth)->value);	
}

void CWSettings::change_difficulty(gint d)
{
	if (debug > 1) g_print("CWSettings::change_difficulty to %d\n",d);
	_settings->_difficulty = d;
	update_difficulty_label();
	gnome_property_box_changed (GNOME_PROPERTY_BOX(_window));
}

void CWSettings::change_height(void)
{
	if (debug > 1) g_print("CWSettings::change_height\n");
	gnome_property_box_changed (GNOME_PROPERTY_BOX(_window));
	_settings->_h = ((gint)GTK_ADJUSTMENT(_adj_height)->value);	
}

void CWSettings::change_width(void)
{
	if (debug > 1) g_print("CWSettings::change_width\n");
	gnome_property_box_changed (GNOME_PROPERTY_BOX(_window));
	_settings->_w = ((gint)GTK_ADJUSTMENT(_adj_width)->value);	
}

void CWSettings::free_memory(void)
{
	if (debug) g_print("CWSettings::free_memory\n");
	delete _settings;	
}

void CWSettings::init(GtkWidget *parent)
{
	if (debug) g_print("CWSettings::init()\n");
	_parent = parent;	
	*_settings = *settings;
}

void CWSettings::show()
{
	if (debug) g_print("CWSettings::show()\n");
	GtkWidget *tab;
	GtkWidget *box1;
	GtkWidget *table;
	GtkWidget *lbl;
	GtkWidget *scale;
	GtkWidget *diff_buttons;
	GtkWidget *diff_box;
	
	// create window
	_window = gnome_property_box_new ();
	gnome_dialog_set_parent (GNOME_DIALOG(_window), GTK_WINDOW(_parent));
	gtk_signal_connect (GTK_OBJECT (_window), "destroy",
		GTK_SIGNAL_FUNC(gtk_widget_destroyed), _window);

	// settings tab //////////////////////////////////
	tab = gtk_label_new (_("Settings"));

	box1 = gtk_vbox_new(FALSE, 5);
    
	table = gtk_table_new(3,2,FALSE);
//	gtk_table_set_col_spacings(GTK_TABLE(table), 10);
	gtk_container_border_width (GTK_CONTAINER(table), GNOME_PAD_SMALL);
	
	// width
	lbl = gtk_label_new(_("World Width"));
	gtk_table_attach(GTK_TABLE(table),lbl,0,1,0,1,
		GTK_FILL, GTK_FILL,
//		GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL,
		GNOME_PAD_SMALL, GNOME_PAD_SMALL);
	_adj_width = gtk_adjustment_new(_settings->_w,
		MIN_WORLD_WIDTH, MAX_WORLD_WIDTH, 1.0, 1.0, 0);
	scale = gtk_hscale_new(GTK_ADJUSTMENT(_adj_width));
	gtk_scale_set_digits(GTK_SCALE(scale),0);
	gtk_scale_set_value_pos(GTK_SCALE(scale), GTK_POS_LEFT);
//	gtk_widget_set_usize(scale,200,20); 
	gtk_table_attach(GTK_TABLE(table),scale,1,2,0,1,
		GTK_FILL | GTK_EXPAND, GTK_FILL,
		GNOME_PAD_SMALL, GNOME_PAD_SMALL);
	gtk_signal_connect(GTK_OBJECT(_adj_width),"value_changed",
		GTK_SIGNAL_FUNC(cb_width_value_changed), this);

	// height
	lbl = gtk_label_new(_("World Height"));
	gtk_table_attach(GTK_TABLE(table),lbl,0,1,1,2,
		GTK_FILL, GTK_FILL,
		GNOME_PAD_SMALL, GNOME_PAD_SMALL);
	_adj_height = gtk_adjustment_new(_settings->_h,
		MIN_WORLD_HEIGHT, MAX_WORLD_HEIGHT, 1.0, 1.0, 0);
	scale = gtk_hscale_new(GTK_ADJUSTMENT(_adj_height));
	gtk_scale_set_digits(GTK_SCALE(scale),0);
	gtk_scale_set_value_pos(GTK_SCALE(scale), GTK_POS_LEFT);
	gtk_table_attach(GTK_TABLE(table),scale,1,2,1,2,
		GTK_FILL, GTK_FILL,
		GNOME_PAD_SMALL, GNOME_PAD_SMALL);
	gtk_signal_connect(GTK_OBJECT(_adj_height),"value_changed",
		GTK_SIGNAL_FUNC(cb_height_value_changed), this);

	// depth
	lbl = gtk_label_new(_("World Depth"));
	gtk_table_attach(GTK_TABLE(table),lbl,0,1,2,3,
		GTK_FILL, GTK_FILL,
		GNOME_PAD_SMALL, GNOME_PAD_SMALL);
	_adj_depth = gtk_adjustment_new(_settings->_d,
		MIN_WORLD_DEPTH, MAX_WORLD_DEPTH, 1.0, 1.0, 0);
	scale = gtk_hscale_new(GTK_ADJUSTMENT(_adj_depth));
	gtk_scale_set_digits(GTK_SCALE(scale),0);
	gtk_scale_set_value_pos(GTK_SCALE(scale), GTK_POS_LEFT);
	gtk_table_attach(GTK_TABLE(table),scale,1,2,2,3,
		GTK_FILL, GTK_FILL,
		GNOME_PAD_SMALL, GNOME_PAD_SMALL);
	gtk_signal_connect(GTK_OBJECT(_adj_depth),"value_changed",
		GTK_SIGNAL_FUNC(cb_depth_value_changed), this);

	gtk_box_pack_start(GTK_BOX(box1), table, TRUE, TRUE, 0);

	// difficulty buttons
	gchar *str = g_strdup_printf(_("The current set of pieces is %s"),
		CSettings::diff_descriptors[_settings->_difficulty]);
	label = gtk_label_new(str);
	g_free(str);
	gtk_box_pack_start(GTK_BOX(box1), label, TRUE, TRUE, 0);
	diff_box = gtk_hbox_new(TRUE, 0);
	diff_buttons = gtk_button_new_with_label(
		CSettings::diff_descriptors[CSettings::diff_EASY]);
	gtk_box_pack_start(GTK_BOX(diff_box), diff_buttons, TRUE, TRUE, 0);  
	gtk_signal_connect(GTK_OBJECT(diff_buttons), "clicked",
		GTK_SIGNAL_FUNC(cb_button_easy_clicked), this);
	diff_buttons = gtk_button_new_with_label(
		CSettings::diff_descriptors[CSettings::diff_MEDIUM]);
	gtk_box_pack_start(GTK_BOX(diff_box), diff_buttons, TRUE, TRUE, 0);  
	gtk_signal_connect(GTK_OBJECT(diff_buttons), "clicked",
		GTK_SIGNAL_FUNC(cb_button_medium_clicked), this);
	diff_buttons = gtk_button_new_with_label(
		CSettings::diff_descriptors[CSettings::diff_HARD]);
	gtk_box_pack_start(GTK_BOX(diff_box), diff_buttons, TRUE, TRUE, 0);  
	gtk_signal_connect(GTK_OBJECT(diff_buttons), "clicked",
		GTK_SIGNAL_FUNC(cb_button_hard_clicked), this);
	gtk_box_pack_start(GTK_BOX(box1), diff_box, TRUE, TRUE, 0);

	gnome_property_box_append_page(GNOME_PROPERTY_BOX(_window), box1, tab);
		

	// bottom & signals
	gtk_signal_connect (GTK_OBJECT(_window), "apply",
		GTK_SIGNAL_FUNC(cb_apply_button_clicked), this);

	gtk_widget_show_all(_window);
}

void CWSettings::update_difficulty_label(void)
{
	gchar *str = g_strdup_printf(_("The current set of pieces is %s"),
		CSettings::diff_descriptors[_settings->_difficulty]);
	gtk_label_set_text(GTK_LABEL(label), str);
	g_free(str);
}

void CWSettings::cb_button_easy_clicked(GtkWidget *widget, gpointer data)
{
	if (debug > DBG_CB) g_print("window_settings::cb_button_easy_clicked\n");
	((CWSettings *)data)->change_difficulty(CSettings::diff_EASY);
}

void CWSettings::cb_button_hard_clicked(GtkWidget *widget, gpointer data)
{
	if (debug > DBG_CB) g_print("window_settings::cb_button_hard_clicked\n");
	((CWSettings *)data)->change_difficulty(CSettings::diff_HARD);
}

void CWSettings::cb_button_medium_clicked(GtkWidget *widget, gpointer data)
{
	if (debug > DBG_CB) g_print("window_settings::cb_button_medium_clicked\n");
	((CWSettings *)data)->change_difficulty(CSettings::diff_MEDIUM);
}

void CWSettings::cb_depth_value_changed(GtkWidget *widget, gpointer data)
{
	if (debug > DBG_CB) g_print("window_settings::cb_depth_value_changed\n");
	((CWSettings *)data)->change_depth();
}

void CWSettings::cb_height_value_changed(GtkWidget *widget, gpointer data)
{
	if (debug > DBG_CB) g_print("window_settings::cb_height_value_changed\n");
	((CWSettings *)data)->change_height();
}

void CWSettings::cb_width_value_changed(GtkWidget *widget, gpointer data)
{
	if (debug > DBG_CB) g_print("window_settings::cb_width_value_changed\n");
	((CWSettings *)data)->change_width();
}

void CWSettings::cb_apply_button_clicked(GtkWidget *widget, gint page_num,
	gpointer data)
{
	if (debug > DBG_CB) g_print("CWSettings::cb_apply_button_clicked(%d)\n",
		page_num);
	if (page_num == -1) ((CWSettings *)data)->apply();
}	
