/* gno3dtet: 3D Tetris game for GNOME
 * Copyright (C) 1999 Sebastien Nicoud <snicoud@rmi.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
*/

#include "common.h"
#include "cgame.h"
#include "whof.h"

// Common globals ////////////////////////////////////////////////////////////

gint		g_nhofl = 0; // number of line to display in the HOF

// Globals ///////////////////////////////////////////////////////////////////

gboolean	g_update = FALSE;	// true when game won, false when show only
CGame		*gw_hof[MAX_NHOFL+2];// hof read from file, stored in the CGame structure
gint		gw_hof_line = 0; // nb of lines in the hof
gboolean	g_player_in_hof = FALSE;	// true when player belongs to the hof
gboolean	g_player_in_shown_hof = FALSE; // true when player belongs to the shown hof
gint		g_player_line = 0; // rank = new line nb
gboolean	g_already_recorded = FALSE;
gint		g_already_recorded_game_index = 0;

// Callbacks /////////////////////////////////////////////////////////////////

/* entry_name_changed
 *  called when user type in the name entry widget
 *  updates the g_hofl global variable
*/
void cb_entry_name_changed(GtkObject *widget, gpointer data)
{
	if (debug > DBG_CB) g_print("whof::cb_entry_name_changed\n");
	mds->set_name(gtk_entry_get_text(GTK_ENTRY(widget)));
}

/* close_dialog
*/
void cb_close_dialog(GtkObject *widget, gpointer data)
{
	if (debug > DBG_CB) g_print("whof::cb_close_dialog\n");
	gnome_dialog_close(GNOME_DIALOG(widget));
}

/* window_name_entry_key_press_event
 * closes the name entry dialog w when return key pressed
*/
gint cb_window_name_entry_key_press_event(GtkWidget *widget,
	GdkEvent *event, gpointer data)
{
	if (debug > DBG_CB) g_print("whof::cb_window_name_entry_key_press_event\n");
	switch ((gint)(((GdkEventKey *)event)->keyval)) {
		case GDK_Return: case GDK_KP_Enter:
		if (debug > DBG_CB) g_print("  key 'return' pressed\n");
		gnome_dialog_close(GNOME_DIALOG(widget));
		break;
		default: // any other key // nothing to do
		break;
}	}

// Functions /////////////////////////////////////////////////////////////////

gint append_hofline()
{
	if (debug) g_print("whof::append_hofline, new line @ %d\n",gw_hof_line);
	gw_hof[gw_hof_line] = g_new0(CGame,1);
	*gw_hof[gw_hof_line] = *mds; // copy
	g_player_line = gw_hof_line;
	gw_hof_line++;
	return g_player_line;
}

gboolean is_game_already_recorded(void)
{
	if (debug) g_print("whof::is_game_already_recorded\n");
	for (gint i=0; i<gw_hof_line; i++)
		if (gw_hof[i]->get_game_id() == mds->get_game_id()) {
			g_already_recorded_game_index = i;
			g_already_recorded = TRUE;
			return TRUE;
		}
	g_already_recorded = FALSE;
	return FALSE;
}

/* read_hof
 *  reads the hof file and updates the hof data structure
 *  returns: 0 -> ok
 *           1 -> no file
 *           - -> error occured
*/
guint read_hof(void)
{
	if (debug) g_print("whof::read_hof\n");
	
	// open hof file
	ifstream fin;
	gchar *filename = g_strdup(GNO3DTET_HOFABSFILE);
	fin.open(filename);
	if (!fin.is_open()) {
		// try current dir
		filename = g_strdup(HOFFILE);
		fin.open(filename);
		if (!fin.is_open()) {
			// try home dir
			filename = g_strdup_printf("%s%s%s",
				g_get_home_dir(), G_DIR_SEPARATOR_S, HOFFILE);
			fin.open(filename);
			if (!fin.is_open()) {
				g_free(filename);
				if (debug) g_print("  read_hof exit elvl 1: can't open file\n");
				return 1;
	}	}	}
	fin.close();
	if (debug) g_print("  read_hof: reading file: >%s<...\n",filename);
	gw_hof_line = 0;
	guint iret = 0;
	CGame temp;
	iret = temp.read_game(filename, TRUE/*from the beginning*/);
	while (iret == 0/*ok*/) {
		gw_hof[gw_hof_line] = g_new0(CGame,1);
		*gw_hof[gw_hof_line] = temp;
		gw_hof_line++;
		if (gw_hof_line >= MAX_NHOFL) break;
		iret = temp.read_game(filename, FALSE/*at current pos*/);
	}
	if (iret == 1/*EOF, it's ok*/) iret = 0;
	if (debug) g_print("  read hof: nb lines read = %d, exit elvl %d\n",
		gw_hof_line,iret);
	g_free(filename);
	return iret;
}
	
/* sort_table
*/
void sort_hof(void) {
	if (debug) g_print("whof::sort_hof\n");

	gint new_line = append_hofline();

	// compute sort-table
	gint sort[MAX_NHOFL+2];
	for (gint i=0;i<gw_hof_line+1;i++) sort[i] = -1;
	for (gint i=0;i<gw_hof_line;i++) {
		if (i == 0)
			// put the first one
			sort[0] = 0;
		else for (gint j=0;j<=i;j++) {
			if (gw_hof[i]->is_better_than(gw_hof[sort[j]])) {
				// move sort table down
				for (gint k=gw_hof_line;k>j;k--) sort[k] = sort[k-1];
				// insert current
				sort[j] = i;
				break; // exit j loop
	}	}	}

	if (new_line != sort[new_line]) {
		g_player_in_hof = TRUE;
		if (new_line < g_nhofl) g_player_in_shown_hof = TRUE;
	}

	// get player rank
	for (gint i=0; i<gw_hof_line; i++)
		if (sort[i] == new_line) { g_player_line=i; break; }

	// sort hof using sort-table
	CGame *tmphof[MAX_NHOFL+2];
	for (gint i=0;i<gw_hof_line;i++) tmphof[i] = gw_hof[i];
	for (gint i=0;i<gw_hof_line;i++) gw_hof[i] = tmphof[sort[i]];
	return;
}

/* write_hof
 *  saves the hof data structure in the hof file
 *  returns: 0 -> ok
 *           1 -> can't open file
 *           - -> error occured
*/
gint write_hof(void) {

	if (debug) g_print("whof::write_hof\n");
	if (!g_player_in_hof) return 0; // nothing changed => nothing to save
	if (gw_hof_line == 0) return 0; // table is empty
	gint iret = 0;

	// truncate table
	if (gw_hof_line > MAX_NHOFL) gw_hof_line = MAX_NHOFL;

	// look for file name & test
	ofstream fout;
	gchar *filename = g_strdup(GNO3DTET_HOFABSFILE);
	fout.open(filename);
	if (!fout.is_open()) {
		// try current dir
		filename = g_strdup(HOFFILE);
		fout.open(filename);
		if (!fout.is_open()) {
			// try home dir
			filename = g_strdup_printf("%s%s%s",
				g_get_home_dir(), G_DIR_SEPARATOR_S, HOFFILE);
			fout.open(filename);
			if (!fout.is_open()) {
				g_free(filename);
				return 1;
	}	}	}
	fout.close();
	if (debug) g_print("  write_hof: writing to >%s<...\n",filename);
	for (gint i=0; i<gw_hof_line; i++) {
			iret = gw_hof[i]->write_game(filename, i==0? TRUE:FALSE);
			if (iret) break; // halt writing
	}
	if (debug) g_print("  write_hof, exit elvl %d\n",iret);
	g_free(filename);
	return iret;
}

/* player enter hof
 * returns : true: the player enters the hof
 *           false: he doesn't
 * the hof DS is supposed sorted
*/
gboolean player_enters_hof(void) {
	if (debug) g_print("window_hof::player enters hof: ");
	if (gw_hof_line < MAX_NHOFL) {
		g_player_in_hof = TRUE;
		if (debug) g_print("yes\n");
		return TRUE; // Hof not full yet => enter whatever the score
	}
	g_player_in_hof = mds->is_better_than(gw_hof[gw_hof_line-1]);
		// player is better than last record
	if (debug) if (g_player_in_hof) g_print("yes\n"); else g_print("no\n");
	if (!g_player_in_hof) g_player_in_shown_hof = FALSE;
	return g_player_in_hof;
}

/* player enter shwon hof
 * returns : true: the player enters the shown hof
 *           false: he doesn't
 * the hof DS is supposed sorted
*/
gboolean player_enters_shown_hof(void) {
	if (debug) g_print("window_hof::player enters shown hof: ");
	if (gw_hof_line < g_nhofl) {
		g_player_in_shown_hof = TRUE;
		if (debug) g_print("yes\n");
		return TRUE;
	}
	g_player_in_shown_hof = mds->is_better_than(gw_hof[g_nhofl-1]); // player is better than last record
	if (debug) if (g_player_in_shown_hof) g_print("yes\n"); else g_print("no\n");
	return g_player_in_shown_hof;
}

void window_name_entry(void)
{
	if (debug) g_print("whof::window_name_entry\n");
	GtkWidget *window_entry;
	GtkWidget *lbl;
	GtkWidget *entry;
	GtkWidget *box;

	window_entry = gnome_dialog_new (_("Gnome 3D Tetris Hall Of Fame"),
		GNOME_STOCK_BUTTON_OK, NULL);
//	if (main_window) gnome_dialog_set_parent (GNOME_DIALOG (window_entry),
//		GTK_WINDOW(main_window));
	gtk_window_set_position(GTK_WINDOW(window_entry),GTK_WIN_POS_CENTER);

	// window handlers
	gtk_signal_connect_object(GTK_OBJECT(window_entry), "delete_event",
		GTK_SIGNAL_FUNC(cb_close_dialog), GTK_OBJECT(window_entry));
	gtk_signal_connect_object(GTK_OBJECT(window_entry), "destroy",
		GTK_SIGNAL_FUNC(cb_close_dialog), GTK_OBJECT(window_entry));

	// stuff
	box = GNOME_DIALOG(window_entry)->vbox;
	lbl=gtk_label_new(_("Congratulation !\n \
		You're entering the Gnome 3D Tetris Hall Of Fame.\n\
		Please enter your name\n"));
	gtk_box_pack_start(GTK_BOX(box), lbl, FALSE, FALSE, 0);
	entry = gtk_entry_new_with_max_length(MAX_NAME_LEN);
	if (mds->get_name() != NULL)
		gtk_entry_set_text(GTK_ENTRY(entry),mds->get_name());
	gtk_box_pack_start(GTK_BOX(box), entry, FALSE, FALSE, 0);
	gtk_signal_connect(GTK_OBJECT(entry), "changed",
		GTK_SIGNAL_FUNC(cb_entry_name_changed), NULL);

	gnome_dialog_button_connect_object(GNOME_DIALOG(window_entry),
		0/*close*/, GTK_SIGNAL_FUNC(cb_close_dialog), GTK_OBJECT(window_entry));
	gtk_signal_connect(GTK_OBJECT(window_entry), "key_press_event",
		GTK_SIGNAL_FUNC(cb_window_name_entry_key_press_event),NULL);

	gtk_widget_show_all(window_entry);
	gnome_dialog_run_and_close (GNOME_DIALOG (window_entry));
	if (debug) g_print("  window_name_entry, name = >%s<\n",mds->get_name());
}

/* window_hof
 *  create the hof window
 *  the HOF isn't a modal window
 *  the hof DB is supposed to be already read
*/
void window_hof() {
	if (debug) g_print("whof::window_hof\n");

	GtkWidget *window;
	GtkWidget *table;
	GtkWidget *box1, *box2;
	GtkWidget *separator;
	GtkWidget *lbl;

	gboolean player_found_code_but_not_shown_hof = FALSE;
	gint nb_to_display = g_nhofl;
	if (gw_hof_line < g_nhofl) nb_to_display = gw_hof_line;
	if (g_update) {
		if (!g_player_in_hof || !g_player_in_shown_hof) {
			player_found_code_but_not_shown_hof = TRUE;
	}	}
	if (debug > 1) g_print("    update tag = %d\n",g_update);
	if (debug > 1) g_print("    already recorded = %d\n",g_already_recorded);
	if (debug) g_print("    showing %d lines\n",nb_to_display);
	if (debug > 1) g_print("    player in hof = %d, in shown = %d\n",
		g_player_in_hof, g_player_in_shown_hof);
	if (debug) g_print("    player found code but not in shown hof = %d\n",
		player_found_code_but_not_shown_hof);

	// create the hof window
	window = gnome_dialog_new (_("Gnome 3D Tetris Hall Of Fame"),
		GNOME_STOCK_BUTTON_CLOSE, NULL);
	gtk_window_set_position(GTK_WINDOW(window),GTK_WIN_POS_CENTER);

	// window handlers
	gtk_signal_connect_object(GTK_OBJECT(window), "delete_event",
		GTK_SIGNAL_FUNC(cb_close_dialog), GTK_OBJECT(window));
	gtk_signal_connect_object(GTK_OBJECT(window), "destroy",
		GTK_SIGNAL_FUNC(cb_close_dialog), GTK_OBJECT(window));

	// window container
	box1 = GNOME_DIALOG(window)->vbox;

	if (player_found_code_but_not_shown_hof) {
		lbl = gtk_label_new(_("Congratulations!\n \
			You played well.\n \
			Unfortunately, you can't enter the Hall Of Fame :-("));
		gtk_box_pack_start(GTK_BOX(box1), lbl, FALSE, FALSE, 0);
	}
	if (g_update) {
		gchar *s = g_strdup_printf(_("You are ranked: %d"),g_player_line+1);
		lbl = gtk_label_new(s);
		g_free(s);
		gtk_box_pack_start(GTK_BOX(box1), lbl, FALSE, FALSE, 0); 
	}
	if (g_already_recorded) {
		gchar *s = g_strdup_printf(_("Congratulations!\n \
			You played well,\n \
			but the game you just completed has already been recorded on\n%s by %s"),
			gw_hof[g_already_recorded_game_index]->stringify_date(),
			gw_hof[g_already_recorded_game_index]->stringify_name());
		lbl = gtk_label_new(s);
		g_free(s);
		gtk_box_pack_start(GTK_BOX(box1), lbl, FALSE, FALSE, 0); 
	}

	// table
	table = gtk_table_new(nb_to_display+2,11,FALSE);
	gtk_table_set_col_spacings(GTK_TABLE(table),10);
	gtk_box_pack_start(GTK_BOX(box1), table, FALSE, FALSE, 0);

	// first line = labels
	lbl = gtk_label_new(_("Rank"));
	gtk_table_attach_defaults(GTK_TABLE(table),lbl,0,1,0,1);
	lbl = gtk_label_new(_("Score"));
	gtk_table_attach_defaults(GTK_TABLE(table),lbl,1,2,0,1);
	lbl = gtk_label_new(_("Difficulty"));
	gtk_table_attach_defaults(GTK_TABLE(table),lbl,2,3,0,1);
	lbl = gtk_label_new(_("Level"));
	gtk_table_attach_defaults(GTK_TABLE(table),lbl,3,4,0,1);
	lbl = gtk_label_new(_("Layers"));
	gtk_table_attach_defaults(GTK_TABLE(table),lbl,4,5,0,1);
	lbl = gtk_label_new(_("Depth"));
	gtk_table_attach_defaults(GTK_TABLE(table),lbl,5,6,0,1);
	lbl = gtk_label_new(_("Width"));
	gtk_table_attach_defaults(GTK_TABLE(table),lbl,6,7,0,1);
	lbl = gtk_label_new(_("Height"));
	gtk_table_attach_defaults(GTK_TABLE(table),lbl,7,8,0,1);
	lbl = gtk_label_new(_("Time"));
	gtk_table_attach_defaults(GTK_TABLE(table),lbl,8,9,0,1);
	lbl = gtk_label_new(_("Name"));
	gtk_table_attach_defaults(GTK_TABLE(table),lbl,9,10,0,1);
	lbl = gtk_label_new(_("Date"));
	gtk_table_attach_defaults(GTK_TABLE(table),lbl,10,11,0,1);

	// separator
	separator = gtk_hseparator_new();
	gtk_table_attach_defaults(GTK_TABLE(table),separator,0,11,1,2);

	// all the lines
	gchar *str = NULL;
	for (gint i=0; i<nb_to_display; i++) {
		gchar *s = g_strdup_printf("%d",i+1);
		lbl = gtk_label_new(s);
		g_free(s);
		gtk_table_attach_defaults(GTK_TABLE(table),lbl,0,1,i+2,i+3);
		str = gw_hof[i]->stringify_score();
		lbl = gtk_label_new(str);
		gtk_table_attach_defaults(GTK_TABLE(table),lbl,1,2,i+2,i+3);
		g_free(str);
		str = gw_hof[i]->stringify_difficulty();
		lbl = gtk_label_new(str);
		gtk_table_attach_defaults(GTK_TABLE(table),lbl,2,3,i+2,i+3);
		g_free(str);
		str = gw_hof[i]->stringify_level();
		lbl = gtk_label_new(str);
		gtk_table_attach_defaults(GTK_TABLE(table),lbl,3,4,i+2,i+3);
		g_free(str);
		str = gw_hof[i]->stringify_layers();
		lbl = gtk_label_new(str);
		gtk_table_attach_defaults(GTK_TABLE(table),lbl,4,5,i+2,i+3);
		g_free(str);
		str = gw_hof[i]->stringify_depth();
		lbl = gtk_label_new(str);
		gtk_table_attach_defaults(GTK_TABLE(table),lbl,5,6,i+2,i+3);
		g_free(str);
		str = gw_hof[i]->stringify_width();
		lbl = gtk_label_new(str);
		gtk_table_attach_defaults(GTK_TABLE(table),lbl,6,7,i+2,i+3);
		g_free(str);
		str = gw_hof[i]->stringify_height();
		lbl = gtk_label_new(str);
		gtk_table_attach_defaults(GTK_TABLE(table),lbl,7,8,i+2,i+3);
		g_free(str);
		str = gw_hof[i]->stringify_timer();
		lbl = gtk_label_new(str);
		gtk_table_attach_defaults(GTK_TABLE(table),lbl,8,9,i+2,i+3);
		g_free(str);
		str = gw_hof[i]->stringify_name();
		lbl = gtk_label_new(str);
		gtk_table_attach_defaults(GTK_TABLE(table),lbl,9,10,i+2,i+3);
		g_free(str);
		str = gw_hof[i]->stringify_date();
		lbl = gtk_label_new(str);
		gtk_table_attach_defaults(GTK_TABLE(table),lbl,10,11,i+2,i+3);
		g_free(str);
	}
	// show player
	if (player_found_code_but_not_shown_hof) {
		gchar *s = g_strdup_printf("%d",g_player_line+1);
		lbl = gtk_label_new(s);
		g_free(s);
		gtk_table_attach_defaults(GTK_TABLE(table),lbl,
			0,1,nb_to_display+2,nb_to_display+3);
		str = mds->stringify_score();
		lbl = gtk_label_new(str);
		gtk_table_attach_defaults(GTK_TABLE(table),lbl,
			1,2,nb_to_display+2,nb_to_display+3);
		g_free(str);
		str = mds->stringify_difficulty();
		lbl = gtk_label_new(str);
		gtk_table_attach_defaults(GTK_TABLE(table),lbl,
			2,3,nb_to_display+2,nb_to_display+3);
		g_free(str);
		str = mds->stringify_level();
		lbl = gtk_label_new(str);
		gtk_table_attach_defaults(GTK_TABLE(table),lbl,
			3,4,nb_to_display+2,nb_to_display+3);
		g_free(str);
		str = mds->stringify_layers();
		lbl = gtk_label_new(str);
		gtk_table_attach_defaults(GTK_TABLE(table),lbl,
			4,5,nb_to_display+2,nb_to_display+3);
		g_free(str);
		str = mds->stringify_depth();
		lbl = gtk_label_new(str);
		gtk_table_attach_defaults(GTK_TABLE(table),lbl,
			5,6,nb_to_display+2,nb_to_display+3);
		g_free(str);
		str = mds->stringify_width();
		lbl = gtk_label_new(str);
		gtk_table_attach_defaults(GTK_TABLE(table),lbl,
			6,7,nb_to_display+2,nb_to_display+3);
		g_free(str);
		str = mds->stringify_height();
		lbl = gtk_label_new(str);
		gtk_table_attach_defaults(GTK_TABLE(table),lbl,
			7,8,nb_to_display+2,nb_to_display+3);
		g_free(str);
		str = mds->stringify_timer();
		lbl = gtk_label_new(str);
		gtk_table_attach_defaults(GTK_TABLE(table),lbl,
			8,9,nb_to_display+2,nb_to_display+3);
		g_free(str);
		lbl = gtk_label_new("YOU ARE HERE");
		gtk_table_attach_defaults(GTK_TABLE(table),lbl,
			9,10,nb_to_display+2,nb_to_display+3);
		str = mds->stringify_date();
		lbl = gtk_label_new(str);
		gtk_table_attach_defaults(GTK_TABLE(table),lbl,
			10,11,nb_to_display+2,nb_to_display+3);
		g_free(str);
	}
	gtk_widget_show_all(window);
	gnome_dialog_button_connect_object(GNOME_DIALOG(window), 0/*close*/,
		GTK_SIGNAL_FUNC(cb_close_dialog), GTK_OBJECT(window));
	
	// free memory
	for (gint i=0; i<gw_hof_line; i++)
		if (gw_hof[i] != NULL) {
			g_free(gw_hof[i]);
			gw_hof[i] = NULL;
		}
	return;
}

void call_hof(gboolean update)
{
	if (debug) g_print("whof::call_hof (update=%d), show %d lines\n",
		update, g_nhofl);
	// init globals
	g_update = update;
	g_already_recorded = FALSE;
	for (gint i=0; i<MAX_NHOFL+2; i++) gw_hof[i] = NULL;

	// setup game date
	if (update) mds->compute_date();
	
  	if (read_hof()) return;
	if (g_update) {
		if (is_game_already_recorded()) {
			// game id is already in the existing hof
			// don't changed anything in the hof
			g_update = FALSE;
		} else {
			if (player_enters_hof()/*updates g_player_in_hof*/) {
				if (player_enters_shown_hof()) window_name_entry();
				if (read_hof()) return;
				sort_hof();
				if (write_hof()) return;
			} else {
				gint new_line = append_hofline();
			}
		}
	}   
	window_hof();
	if (debug) g_print("whof: exiting call_hof()\n");
	return;
}
